@@ -425,10 +425,10 @@ def reindex(adata, genes, chunk_size=CHUNK_SIZE):
     if len(idx) == len(genes):
         adata = adata[:, genes]
     else:
-        new_X = scipy.sparse.csr_matrix((adata.shape[0], len(genes)))
+        new_X = scipy.sparse.lil_matrix((adata.shape[0], len(genes)))
         for i in range(new_X.shape[0]//chunk_size+1):
             new_X[i*chunk_size:(i+1)*chunk_size, idx] = adata[i*chunk_size:(i+1)*chunk_size, genes[idx]].X
-        adata = AnnData(new_X, obs=adata.obs, var={'var_names':genes}) 
+        adata = AnnData(new_X.tocsr(), obs=adata.obs, var={'var_names':genes}) 
     return adata
 
 
@@ -564,6 +564,7 @@ def load_data(
     if 'batch' not in adata.obs:
         adata.obs['batch'] = 'batch'
     adata.obs['batch'] = adata.obs['batch'].astype('category')
+    if log: log.info('There are {} batches under batch_name: {}'.format(len(adata.obs['batch'].cat.categories), batch_name))
     
     if isinstance(n_top_features, str):
         if os.path.isfile(n_top_features):
