@@ -13,6 +13,7 @@ import numpy as np
 import os
 import scanpy as sc
 from anndata import AnnData
+from typing import Union, List
 
 from .data import load_data
 from .net.vae import VAE
@@ -23,37 +24,35 @@ from .plot import embedding
 
 
 def SCALEX(
-        data_list=None, 
-        batch_categories=None,
-        profile='RNA',
-        join='inner', 
-        batch_key='batch', 
-        batch_name='batch',
-        min_features=600, 
-        min_cells=3, 
-        target_sum=None,
-        n_top_features=None, 
-        processed=False,
-        batch_size=64, 
-        lr=2e-4, 
-        max_iteration=30000,
-        seed=124, 
-        gpu=0, 
-        outdir='output/', 
-        projection=None,
-        repeat=False,
-        impute=None, 
-        chunk_size=20000,
-        ignore_umap=False,
-        verbose=False,
-        assess=False,
-        show=True,
-        eval=False,
-        test_list=None,
-        test_batch_categories=None,
-    ):
+        data_list:Union[str, AnnData, List]=None, 
+        batch_categories:List=None,
+        profile:str='RNA',
+        batch_name:str='batch',
+        min_features:int=600, 
+        min_cells:int=3, 
+        target_sum:int=None,
+        n_top_features:int=None,
+        join:str='inner', 
+        batch_key:str='batch',  
+        processed:bool=False,
+        batch_size:int=64, 
+        lr:float=2e-4, 
+        max_iteration:int=30000,
+        seed:int=124, 
+        gpu:int=0, 
+        outdir:str=None, 
+        projection:str=None,
+        repeat:bool=False,
+        impute:str=None, 
+        chunk_size:int=20000,
+        ignore_umap:bool=False,
+        verbose:bool=False,
+        assess:bool=False,
+        show:bool=True,
+        eval:bool=False,
+    ) -> AnnData:
     """
-    Single-Cell integrative Analysis via Latent feature Extraction
+    Online single-cell data integration through projecting heterogeneous datasets into a common cell-embedding space
     
     Parameters
     ----------
@@ -63,10 +62,6 @@ def SCALEX(
         Categories for the batch annotation. By default, use increasing numbers.
     profile
         Specify the single-cell profile, RNA or ATAC. Default: RNA.
-    join
-        Use intersection ('inner') or union ('outer') of variables of different batches. 
-    batch_key
-        Add the batch annotation to obs using this key. By default, batch_key='batch'.
     batch_name
         Use this annotation in obs as batches for training model. Default: 'batch'.
     min_features
@@ -75,6 +70,10 @@ def SCALEX(
         Filtered out genes that are detected in less than min_cells. Default: 3.
     n_top_features
         Number of highly-variable genes to keep. Default: 2000.
+    join
+        Use intersection ('inner') or union ('outer') of variables of different batches. 
+    batch_key
+        Add the batch annotation to obs using this key. By default, batch_key='batch'.
     batch_size
         Number of samples per batch to load. Default: 64.
     lr
@@ -124,9 +123,13 @@ def SCALEX(
     else:
         device='cpu'
     
-    outdir = outdir+'/'
-    os.makedirs(outdir+'/checkpoint', exist_ok=True)
-    log = create_logger('', fh=outdir+'log.txt')
+    if outdir:
+        outdir = outdir+'/'
+        os.makedirs(outdir+'/checkpoint', exist_ok=True)
+        log = create_logger('SCALEX', fh=outdir+'log.txt', overwrite=True)
+    else:
+        log = create_logger('SCALEX')
+
     if not projection:
         adata, trainloader, testloader = load_data(
             data_list, batch_categories, 
@@ -144,7 +147,7 @@ def SCALEX(
             log=log,
         )
         
-        early_stopping = EarlyStopping(patience=10, checkpoint_file=outdir+'/checkpoint/model.pt')
+        early_stopping = EarlyStopping(patience=10, checkpoint_file=outdir+'/checkpoint/model.pt' if outdir else None)
         x_dim, n_domain = adata.shape[1], len(adata.obs['batch'].cat.categories)
         
         # model config
@@ -153,7 +156,7 @@ def SCALEX(
 
         model = VAE(enc, dec, n_domain=n_domain)
         
-        log.info('model\n'+model.__repr__())
+        # log.info('model\n'+model.__repr__())
         model.fit(
             trainloader, 
             lr=lr, 
@@ -162,7 +165,8 @@ def SCALEX(
             early_stopping=early_stopping, 
             verbose=verbose,
         )
-        torch.save({'n_top_features':adata.var.index, 'enc':enc, 'dec':dec, 'n_domain':n_domain}, outdir+'/checkpoint/config.pt')     
+        if outdir:
+            torch.save({'n_top_features':adata.var.index, 'enc':enc, 'dec':dec, 'n_domain':n_domain}, outdir+'/checkpoint/config.pt')
     else:
         state = torch.load(projection+'/checkpoint/config.pt')
         n_top_features, enc, dec, n_domain = state['n_top_features'], state['enc'], state['dec'], state['n_domain']
@@ -189,7 +193,7 @@ def SCALEX(
     adata.obsm['latent'] = model.encodeBatch(testloader, device=device, eval=eval) # save latent rep
     if impute:
         adata.layers['impute'] = model.encodeBatch(testloader, out='impute', batch_id=impute, device=device, eval=eval)
-    log.info('Output dir: {}'.format(outdir))
+    # log.info('Output dir: {}'.format(outdir))
     
     model.to('cpu')
     del model
@@ -201,23 +205,32 @@ def SCALEX(
             batch_key='projection', 
             index_unique=None
         )
-    adata.write(outdir+'adata.h5ad', compression='gzip')  
+
+    if outdir is not None:
+        adata.write(outdir+'adata.h5ad', compression='gzip')  
+
     if not ignore_umap: #and adata.shape[0]<1e6:
         log.info('Plot umap')
         sc.pp.neighbors(adata, n_neighbors=30, use_rep='latent')
         sc.tl.umap(adata, min_dist=0.1)
         sc.tl.leiden(adata)
+        adata.obsm['X_scalex_umap'] = adata.obsm['X_umap']
         
         # UMAP visualization
-        sc.settings.figdir = outdir
         sc.set_figure_params(dpi=80, figsize=(3,3))
-        cols = ['batch', 'celltype', 'leiden']
+        cols = ['batch', 'celltype', 'cell_type', 'leiden']
         color = [c for c in cols if c in adata.obs]
+        if outdir:
+            sc.settings.figdir = outdir
+            save = '.png'
+        else:
+            save = None
+
         if len(color) > 0:
             if projection and (not repeat):
-                embedding(adata, color='leiden', groupby='projection', save='.png', show=show)
+                embedding(adata, color='leiden', groupby='projection', save=save, show=show)
             else:
-                sc.pl.umap(adata, color=color, save='.png', wspace=0.4, ncols=4, show=show)  
+                sc.pl.umap(adata, color=color, save=save, wspace=0.4, ncols=4, show=show)  
         if assess:
             if len(adata.obs['batch'].cat.categories) > 1:
                 entropy_score = batch_entropy_mixing_score(adata.obsm['X_umap'], adata.obs['batch'])
@@ -227,7 +240,8 @@ def SCALEX(
                 sil_score = silhouette_score(adata.obsm['X_umap'], adata.obs['celltype'].cat.codes)
                 log.info("silhouette_score: {:.3f}".format(sil_score))
 
-    adata.write(outdir+'adata.h5ad', compression='gzip')
+    if outdir is not None:
+        adata.write(outdir+'adata.h5ad', compression='gzip')
     
     return adata
         
