@@ -9,9 +9,7 @@
 """
 
 import torch
-from torch.nn import functional as F
 from torch.distributions import Normal, kl_divergence
-import math
 
 def kl_div(mu, var):
     return kl_divergence(Normal(mu, var.sqrt()),
